#!/usr/bin/env python

"A simple file indexer."

import codecs

class Indexer:
    def __init__(self, dict_location, encoding=None):
        self.dict_location = dict_location
        self.encoding = encoding

        # Initialisation.

        self.index = self.make_index()

    def get_index(self):
        return self.index

    def make_index(self):

        "Return a dictionary containing an index structure for the dict."

        if self.encoding is None:
            f = open(self.dict_location)
        else:
            f = codecs.open(self.dict_location, encoding=self.encoding)
        s = f.read()
        f.close()

        tokens = s.split()
        index = {}

        for token in tokens:
            slot = index
            for c in token:
                if not slot.has_key(c):
                    slot[c] = {}, []
                slot, words = slot[c]

            if token not in words:
                words.append(token)

        return index

class Searcher:
    def __init__(self, index):
        self.index = index

    def find(self, pattern):

        "Find words beginning with the given 'pattern'."

        slot = self.index
        words = []

        for c in pattern:
            if not slot.has_key(c):
                return []
            slot, words = slot[c]

        results = []
        results += words
        results += self.get_all_words(slot)
        return results

    def get_all_words(self, slot):

        "Get all words under the given index 'slot'."

        all_words = []
        keys = slot.keys()
        keys.sort()
        for c in keys:
            this_slot, words = slot[c]
            all_words += words
            all_words += self.get_all_words(this_slot)
        return all_words

def update(index1, index2):
    for key in index2.keys():
        if not index1.has_key(key):
            index1[key] = index2[key]
        else:
            slot1, words1 = index1[key]
            slot2, words2 = index2[key]
            for word in words2:
                if not word in words1:
                    words1.append(word)
            update(slot1, slot2)

# vim: tabstop=4 expandtab shiftwidth=4
